# -*- coding: utf-8 -*-

__name__ = 'pontoon'
__description__ = 'A Python CLI for Digital Ocean'
__version__ = '0.1.2'
__author__ = 'Ross Duggan'
__author_email__ = 'ross.duggan@acm.org'
__url__ = 'https://github.com/duggan/pontoon'
__copyright__ = 'Copyright Ross Duggan 2013'

from .cache import cache
from .log import debug

from .pontoon import Pontoon
from .command import Command
from .pontoon import ClientException
from .exceptions import *
